unit fdbtPluginMain;

interface

uses Windows, SysUtils, Classes, TagEditorLibConsts, TagEditorLibAddinConsts,
  fdbtPluginUtils, fdbtPlugin, TTAfile;



var
  // W
  GenreListData: TTagListData;        

const
  FDBTP_INFO: array[0..FDBT_AUTHOR] of string = (
        '1.0',
        'TTA',
        '.tta,.mka',
        '̂炾'
        );


  FDBT_FIELD_LENGTH      = 'length';
  FDBT_FIELD_BITRATE     = 'bitrate';
  FDBT_FIELD_SAMPLERATE  = 'SampleRate';
  FDBT_FIELD_CHANNELS    = 'channels';

  FDBT_FIELD_RATIO       = 'Ratio';


  FDBTP_FIELDCOUNT = 13;
  FDBTP_FIELDS: array[0..FDBTP_FIELDCOUNT-1] of TTagField = (
        (
        Name:        FDBT_FIELD_TITLE;
        DisplayName: '^Cg';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_ARTIST;
        DisplayName: 'A[eBXg';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_ALBUM;
        DisplayName: 'Ao';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_TRACK;
        DisplayName: 'gbN';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_DURATION;
        DisplayName: 't';
        ValueType:   FDBT_VALUE_TYPE_DOUBLE;
        Editable:    False;      Visible: False;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_GENRE;
        DisplayName: 'W';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_YEAR;
        DisplayName: '';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_COMMENT;
        DisplayName: 'Rg';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),

        (
        Name:        FDBT_FIELD_LENGTH;
        DisplayName: '';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;     Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_SAMPLERATE;
        DisplayName: 'Tv[g';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_CHANNELS;
        DisplayName: '`l';
        ValueType:   FDBT_VALUE_TYPE_INTEGER;
        Editable:    False;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_BITRATE;
        DisplayName: 'rbg[g';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;     Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_RATIO;
        DisplayName: 'k';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;      Visible: True;
        Data:        nil;
        )
        );


type

TFdbtPluginMain = class(TFdbtPlugin)
private
  FAddinMenu: TAddinMenu;
  FGetTrackDataInfo: TGetTrackDataInfo;
public
  TrackTools: TTrackDataInfo;
  function Initialize(info: PFdbtPluginInitInfo): integer; override;
  function GetLibInfo(InfoNo: Integer; Buffer: PChar; Size: Integer): Integer; override;
  function ShowConfig: integer; override;
  function GetTagInfo(FileName: PWideChar; Tag: PTagInfo; Size: integer): integer; override;
  function SetTagInfo(FileName: PWideChar; Tag: PTagInfo): integer; override;
  function GetTagFieldInfo(fl: PTagFieldList; size: integer): integer; override;

  function AddinMenu(place: integer; Identify: PChar; DisplayName: PChar; ProcExe: TAddinProc; ImageType: integer; Image: cardinal): integer;
  function GetTrackDataInfo: TTrackDataInfo;
end;


implementation



function TFdbtPluginMain.Initialize(info: PFdbtPluginInitInfo): integer;
begin
  inherited Initialize(info);
  @FAddinMenu:=InitInfo.GetService('AddinMenu');
  @FGetTrackDataInfo:=InitInfo.GetService('GetTrackDataInfo');

  TrackTools:=GetTrackDataInfo();

  Result:=FDBT_SUCCESS;
end;


function TFdbtPluginMain.GetLibInfo(InfoNo: Integer; Buffer: PChar; Size: Integer): Integer;
var
  PlugInfo : string;
begin
  PlugInfo:=FDBTP_INFO[InfoNo];
  lstrcpyn(Buffer, PChar(PlugInfo), Size);
  Result:=Length(Buffer);
end;

function TFdbtPluginMain.ShowConfig: integer;
begin
  Result:=FDBT_NOTIMPLEMENTED;
end;

function TFdbtPluginMain.GetTagInfo(FileName: PWideChar; Tag: PTagInfo; Size: integer): integer;
var
  tg: TTTA;
begin
  tg:=TTTA.Create;

  if tg.ReadFromFile(FileName) then
  begin                                                 
    SetTagValue(Tag^[0], FDBT_FIELD_TITLE,    UTF8Decode(tg.APEtag.SeekField('Title')));
    SetTagValue(Tag^[1], FDBT_FIELD_ARTIST,   UTF8Decode(tg.APEtag.SeekField('Artist')));
    SetTagValue(Tag^[2], FDBT_FIELD_ALBUM,    UTF8Decode(tg.APEtag.SeekField('Album')));
    SetTagValue(Tag^[3], FDBT_FIELD_TRACK,    UTF8Decode(tg.APEtag.SeekField('Track')));
    SetTagValue(Tag^[4], FDBT_FIELD_YEAR,     UTF8Decode(tg.APEtag.SeekField('Year')));
    SetTagValue(Tag^[5], FDBT_FIELD_GENRE,    UTF8Decode(tg.APEtag.SeekField('Genre')));
    SetTagValue(Tag^[6], FDBT_FIELD_COMMENT,  UTF8Decode(tg.APEtag.SeekField('Comment')));
    SetTagValue(Tag^[7], FDBT_FIELD_DURATION, tg.Duration);
                                                        
    SetTagValue(Tag^[8],  FDBT_FIELD_CHANNELS,    tg.Channels);
    SetTagValue(Tag^[9],  FDBT_FIELD_LENGTH,      DurationToStr(Round(tg.Duration), False));
    SetTagValue(Tag^[10], FDBT_FIELD_BITRATE,     Format('%f kbit/s', [(tg.Bitrate/1000)]));
    SetTagValue(Tag^[11], FDBT_FIELD_SAMPLERATE,  Format('%d hz', [tg.SampleRate]));
    SetTagValue(Tag^[12], FDBT_FIELD_RATIO,       Format('%f %%', [tg.Ratio]));

    Result:=14;
  end else
    Result:=0;

  tg.Free;
end;

function TFdbtPluginMain.SetTagInfo(FileName: PWideChar; Tag: PTagInfo): integer;
var
  tg: TTTA;
  i,c: integer;
  fld, v: string;
begin

  tg:=TTTA.Create;
  c:=Length(Tag^);

  for i:=0 to c-1 do
  begin
    fld := Tag^[i].Field;
    v   := UTF8Encode(Value2String(Tag^[i].Value));

    if fld=FDBT_FIELD_TITLE   then tg.APEtag.AppendField('Title', v);
    if fld=FDBT_FIELD_ARTIST  then tg.APEtag.AppendField('Artist', v);
    if fld=FDBT_FIELD_ALBUM   then tg.APEtag.AppendField('Album', v);
    if fld=FDBT_FIELD_TRACK   then tg.APEtag.AppendField('Track', v);
    if fld=FDBT_FIELD_GENRE   then tg.APEtag.AppendField('Genre', v);
    if fld=FDBT_FIELD_YEAR    then tg.APEtag.AppendField('Year', v);
    if fld=FDBT_FIELD_COMMENT then tg.APEtag.AppendField('Comment', v);
  end;

  if tg.APETag.WriteTagInFile(FileName) then
    Result:=FDBT_SUCCESS
  else
    Result:=FDBT_INTERNAL_ERROR;

  tg.Free;
end;

function TFdbtPluginMain.GetTagFieldInfo(fl: PTagFieldList; size: integer): integer;
var
  i: integer;
begin
  for i:=0 to FDBTP_FIELDCOUNT-1 do
    fl^[i]:=FDBTP_FIELDS[i];

  Result:=FDBTP_FIELDCOUNT;
end;

function TFdbtPluginMain.AddinMenu(place: integer; Identify: PChar; DisplayName: PChar; ProcExe: TAddinProc; ImageType: integer; Image: cardinal): integer;
begin
  if Assigned(FAddinMenu) then
    Result:=FAddinMenu(place, Identify, DisplayName, ProcExe, ImageType, Image)
  else
    Result:=FDBT_NOTIMPLEMENTED;
end;

function TFdbtPluginMain.GetTrackDataInfo: TTrackDataInfo;
begin
  if Assigned(FGetTrackDataInfo) then Result:=FGetTrackDataInfo();
end;



initialization


end.
